# 
# Copyright (c) 2022 @enslay (poster at chevybolt.org)
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
import os
import numpy as np
from BoltSolver import Bolt, SolveForShortestTime, SolveForShortestTimeMultipleStops, IntegrateOverSOC, D

def PlotBoltChargeCurves(path="chargeCurves.gnuplot"):
    with open(path, mode="wt", newline="") as f:
        f.write("set term 'epslatex' color size 4in,3in\n")
        f.write("set output 'chargeCurves.tex'\n")
        #f.write("set title 'Chevy Bolt Charge Curves'\n")
        f.write("set xrange [0:1]\n")
        f.write("set xlabel 'SOC'\n")
        f.write("set ylabel 'kW'\n")
        f.write("set xtics 0.1\n")
        f.write("set ytics 10\n")
        f.write("set grid\n")
        f.write("set key left bottom\n")
        f.write("set style line 1 lw 4 lc 'red' lt 1\n")
        f.write("set style line 2 lw 4 lc 'blue' lt 1 dt 3\n")
        f.write("plot '-' using 1:2 w lines ls 1 title 'Chevy Bolt 2020+', '-' using 1:2 w lines ls 2 title 'Chevy Bolt 2017-2019'\n")

        for soc, kw in Bolt(year=2020)[0]:
            f.write(f"{soc} {kw}\n")

        f.write("e\n")

        for soc, kw in Bolt(year=2017)[0]:
            f.write(f"{soc} {kw}\n")

        f.write("e\n")

def PlotOptimalSpeedVersusDistance(path="optimalSpeed.gnuplot", SInitial=0.8, SMargin=0.15):
    C, maxCapacity, maxRange = Bolt(year=2020)
    S, T, dT = IntegrateOverSOC(C, maxCapacity)

    travelDistances = []
    optimalSpeeds = []
    totalTimes = []
    travelTimes = []
    chargeTimes = []
    overallSpeeds = []

    for travelDistance in np.linspace(0, 200, 41):
        v, totalTime, travelTime, chargeTime, destinationSOC, chargeToSOC = SolveForShortestTime(travelDistance, maxCapacity, S, T, dT, SInitial=SInitial, SMargin=SMargin)

        if destinationSOC < 0.0:
            break

        travelDistances.append(travelDistance)
        optimalSpeeds.append(v)
        totalTimes.append(totalTime)
        travelTimes.append(travelTime)
        chargeTimes.append(chargeTime)
        overallSpeeds.append(travelDistance/totalTime if totalTime > 0 else 0)

    with open(path, mode="wt", newline="") as f:
        f.write("set term 'epslatex' color size 5in,3in\n")
        f.write("set output 'optimalSpeed.tex'\n")
        #f.write("set title '2020 Chevy Bolt Optimal Speeds'\n")
        f.write("set xlabel 'Distance (miles)'\n")
        f.write("set ylabel 'Speed (mph)'\n")
        f.write("set y2label 'Time (hours)'\n")
        f.write("set y2tics\n")
        #f.write("set xtics 0.1\n")
        #f.write("set ytics 10\n")
        f.write("set grid\n")
        f.write("set key left top\n")
        f.write("set style line 1 lw 4 lc 'red' lt 1\n")
        f.write("set style line 2 lw 4 lc 'blue' lt 1 dt 2\n")
        f.write("set style line 3 lw 4 lc 'green' lt 1 dt 3\n")
        f.write("set style line 4 lw 4 lc 'purple' lt 1 dt 4\n")
        f.write("plot '-' using 1:2 w lines ls 1 title 'Optimal Speed', '-' using 1:2 axes x1y2 w lines ls 2 title 'Total Time',\\\n")
        f.write("   '-' using 1:2 axes x1y2 w lines ls 3 title 'Travel Time', '-' using 1:2 axes x1y2 w lines ls 4 title 'Charge Time'\n")

        for travelDistance, v in zip(travelDistances, optimalSpeeds):
            f.write(f"{travelDistance} {v}\n")

        f.write("e\n")

        for travelDistance, totalTime in zip(travelDistances, totalTimes):
            f.write(f"{travelDistance} {totalTime}\n")

        f.write("e\n")

        for travelDistance, travelTime in zip(travelDistances, travelTimes):
            f.write(f"{travelDistance} {travelTime}\n")

        f.write("e\n")

        for travelDistance, chargeTime in zip(travelDistances, chargeTimes):
            f.write(f"{travelDistance} {chargeTime}\n")

        f.write("e\n")

def PlotOptimalSpeedSOC(path="optimalSpeedSOC.gnuplot", SInitial=0.8, SMargin=0.15):
    C, maxCapacity, maxRange = Bolt(year=2020)
    S, T, dT = IntegrateOverSOC(C, maxCapacity)

    travelDistances = []
    optimalSpeeds = []
    totalTimes = []
    travelTimes = []
    chargeTimes = []
    overallSpeeds = []
    chargeToSOCs = []
    destinationSOCs = []

    for travelDistance in np.linspace(0, 200, 41):
        v, totalTime, travelTime, chargeTime, destinationSOC, chargeToSOC = SolveForShortestTime(travelDistance, maxCapacity, S, T, dT, SInitial=SInitial, SMargin=SMargin)

        if destinationSOC < 0.0:
            break

        travelDistances.append(travelDistance)
        optimalSpeeds.append(v)
        totalTimes.append(totalTime)
        travelTimes.append(travelTime)
        chargeTimes.append(chargeTime)
        overallSpeeds.append(travelDistance/totalTime if totalTime > 0 else 0)
        destinationSOCs.append(destinationSOC)
        chargeToSOCs.append(chargeToSOC)

    with open(path, mode="wt", newline="") as f:
        f.write("set term 'epslatex' color size 5in,3in\n")
        f.write("set output 'optimalSpeedSOC.tex'\n")
        #f.write("set title '2020 Chevy Bolt SOCs'\n")
        f.write("set xlabel 'Distance (miles)'\n")
        f.write("set ylabel 'SOC'\n")
        f.write("set yrange [0:1]\n")
        f.write("set ytics 0.1\n")
        f.write("set grid\n")
        f.write("set key left bottom\n")
        f.write("set style line 1 lw 4 lc 'red' lt 1\n")
        f.write("set style line 2 lw 4 lc 'blue' lt 1 dt 2\n")
        f.write("set style line 3 lw 4 lc 'green' lt 1 dt 3\n")
        f.write("set style line 4 lw 4 lc 'purple' lt 1 dt 4\n")
        f.write("plot '-' using 1:2 w lines ls 1 title 'Destination SOC', '-' using 1:2 w lines ls 2 title 'Charge SOC'\n")

        for travelDistance, soc in zip(travelDistances, destinationSOCs):
            f.write(f"{travelDistance} {soc}\n")

        f.write("e\n")

        for travelDistance, soc in zip(travelDistances, chargeToSOCs):
            f.write(f"{travelDistance} {soc}\n")

        f.write("e\n")

def PlotOptimalSpeedVersusSpacing(path="optimalSpeedSpacing.gnuplot", SInitial=1.0, SMargin=0.15):
    C, maxCapacity, maxRange = Bolt(year=2020)
    S, T, dT = IntegrateOverSOC(C, maxCapacity)

    travelDistance = 1500

    spacings = []
    optimalSpeeds = []
    totalTimes = []
    travelTimes = []
    chargeTimes = []
    overallSpeeds = []

    for spacing in np.linspace(25, 175, 31):
        fullStops = int(travelDistance // spacing)

        remainder = travelDistance - fullStops * spacing

        travelDistances = [spacing]*fullStops

        if remainder > 0:
            travelDistances += [remainder]

        v, totalTime, travelTime, chargeTime, destinationSOC, chargeToSOC = SolveForShortestTimeMultipleStops(travelDistances, maxCapacity, S, T, dT, SInitial=SInitial, SMargin=SMargin)

        spacings.append(spacing)
        optimalSpeeds.append(v)
        totalTimes.append(totalTime)
        travelTimes.append(travelTime.sum())
        chargeTimes.append(chargeTime.sum())
        overallSpeeds.append(travelDistance/totalTime if totalTime > 0 else 0)

    with open(path, mode="wt", newline="") as f:
        f.write("set term 'epslatex' color size 5in,3in\n")
        f.write("set output 'optimalSpeedSpacing.tex'\n")
        #f.write("set title '2020 Chevy Bolt Optimal Speeds'\n")
        f.write("set xlabel 'Distance per Stop (miles)'\n")
        f.write("set ylabel 'Speed (mph)'\n")
        f.write("set y2label 'Time (hours)'\n")
        f.write("set y2tics\n")
        f.write("set y2range [0:]\n")
        f.write("set grid\n")
        f.write("set key left bottom\n")
        f.write("set style line 1 lw 4 lc 'red' lt 1\n")
        f.write("set style line 2 lw 4 lc 'blue' lt 1 dt 2\n")
        f.write("set style line 3 lw 4 lc 'green' lt 1 dt 3\n")
        f.write("set style line 4 lw 4 lc 'purple' lt 1 dt 4\n")
        f.write("plot '-' using 1:2 w lines ls 1 title 'Optimal Speed', '-' using 1:2 axes x1y2 w lines ls 2 title 'Total Time',\\\n")
        f.write("   '-' using 1:2 axes x1y2 w lines ls 3 title 'Travel Time', '-' using 1:2 axes x1y2 w lines ls 4 title 'Charge Time'\n")

        for spacing, v in zip(spacings, optimalSpeeds):
            f.write(f"{spacing} {v}\n")

        f.write("e\n")

        for spacing, totalTime in zip(spacings, totalTimes):
            f.write(f"{spacing} {totalTime}\n")

        f.write("e\n")

        for spacing, travelTime in zip(spacings, travelTimes):
            f.write(f"{spacing} {travelTime}\n")

        f.write("e\n")

        for spacing, chargeTime in zip(spacings, chargeTimes):
            f.write(f"{spacing} {chargeTime}\n")

        f.write("e\n")

def PlotOptimalSpeedVersusSpacingSOC(path="optimalSpeedSpacingSOC.gnuplot", SInitial=1.0, SMargin=0.15):
    C, maxCapacity, maxRange = Bolt(year=2020)
    S, T, dT = IntegrateOverSOC(C, maxCapacity)

    travelDistance = 1500

    spacings = []
    optimalSpeeds = []
    totalTimes = []
    travelTimes = []
    chargeTimes = []
    overallSpeeds = []
    chargeToSOCs = []

    for spacing in np.linspace(25, 175, 31):
        fullStops = int(travelDistance // spacing)

        remainder = travelDistance - fullStops * spacing

        travelDistances = [spacing]*fullStops

        if remainder > 0:
            travelDistances += [remainder]

        v, totalTime, travelTime, chargeTime, destinationSOC, chargeToSOC = SolveForShortestTimeMultipleStops(travelDistances, maxCapacity, S, T, dT, SInitial=SInitial, SMargin=SMargin)

        spacings.append(spacing)
        optimalSpeeds.append(v)
        totalTimes.append(totalTime)
        travelTimes.append(travelTime.sum())
        chargeTimes.append(chargeTime[-2]) # Second to last charge time
        chargeToSOCs.append(chargeToSOC[-2]) # Second to last charge-to SOC
        overallSpeeds.append(travelDistance/totalTime if totalTime > 0 else 0)

    with open(path, mode="wt", newline="") as f:
        f.write("set term 'epslatex' color size 5in,3in\n")
        f.write("set output 'optimalSpeedSpacingSOC.tex'\n")
        #f.write("set title '2020 Chevy Bolt SOCs'\n")
        f.write("set xlabel 'Distance per Stop (miles)'\n")
        f.write("set ylabel 'SOC'\n")
        f.write("set y2label 'Time (hours)'\n")
        f.write("set y2tics\n")
        f.write("set y2range [0:]\n")
        f.write("set yrange [0:1]\n")
        f.write("set ytics 0.1\n")
        f.write("set grid\n")
        f.write("set key left bottom\n")
        f.write("set style line 1 lw 4 lc 'red' lt 1\n")
        f.write("set style line 2 lw 4 lc 'blue' lt 1 dt 2\n")
        f.write("set style line 3 lw 4 lc 'green' lt 1 dt 3\n")
        f.write("set style line 4 lw 4 lc 'purple' lt 1 dt 4\n")
        f.write("plot '-' using 1:2 w lines ls 1 title 'Charge SOC', '-' using 1:2 axes x1y2 w lines ls 2 title 'Charge Time'\n")

        for spacing, soc in zip(spacings, chargeToSOCs):
            f.write(f"{spacing} {soc}\n")

        f.write("e\n")

        for spacing, chargeTime in zip(spacings, chargeTimes):
            f.write(f"{spacing} {chargeTime}\n")

        f.write("e\n")

def PlotSpeedVersusDistance(path="highSpeed.gnuplot", SInitial=0.8, SMargin=0.15, speed=80.0):
    C, maxCapacity, maxRange = Bolt(year=2020)
    S, T, dT = IntegrateOverSOC(C, maxCapacity)

    chargeStopOverhead = 10.0/60.0

    base=os.path.splitext(os.path.basename(path))[0]

    # Total time spent traveling at speed v to the destination
    def TravelTime(travelDistance):
        return travelDistance/speed

    # Total amount of power consumed traveling
    def TravelPower(travelDistance):
        return D(speed) * TravelTime(travelDistance)

    # The SOC upon reaching the destination
    def DestinationSOC(travelDistance):
        return SInitial - TravelPower(travelDistance)/maxCapacity

    # The SOC to charge to for the return trip
    def ChargeToSOC(travelDistance):
        return min(1.0, SMargin + TravelPower(travelDistance)/maxCapacity)

    # The time needed to charge from DestinationSOC to ChargeToSOC
    def ChargeTime(travelDistance):
        value = np.interp(ChargeToSOC(travelDistance), S, T) - np.interp(DestinationSOC(travelDistance), S, T)

        if value > 0.0:
            return value + chargeStopOverhead

        return 0.0

    # The total time to travel and charge
    def TotalTime(travelDistance):
        return 2.0*TravelTime(travelDistance) + ChargeTime(travelDistance)

    travelDistances = []
    totalTimesDiff = []
    travelTimesDiff = []
    chargeTimesDiff = []
    totalTimesRelDiff = []

    for travelDistance in np.linspace(0, 200, 41):
        v, totalTime, travelTime, chargeTime, destinationSOC, chargeToSOC = SolveForShortestTime(travelDistance, maxCapacity, S, T, dT, SInitial=SInitial, SMargin=SMargin, chargeStopOverhead=chargeStopOverhead)

        totalTimeFixed = TotalTime(travelDistance)
        travelTimeFixed = 2.0*TravelTime(travelDistance)
        chargeTimeFixed = ChargeTime(travelDistance)

        destinationSOCFixed = DestinationSOC(travelDistance)

        if destinationSOCFixed < 0.0 or destinationSOC < 0.0:
            break

        travelDistances.append(travelDistance)
        totalTimesDiff.append((totalTimeFixed - totalTime)*60)
        travelTimesDiff.append((travelTimeFixed - travelTime)*60)
        chargeTimesDiff.append((chargeTimeFixed - chargeTime)*60)
        totalTimesRelDiff.append((totalTimeFixed - totalTime)/totalTime if totalTime > 0.0 else 0.0)

    print(f"Maximum relative round-trip time difference for {speed} mph: {max(totalTimesRelDiff)}")

    with open(path, mode="wt", newline="") as f:
        #f.write("set term 'epslatex' color size 5in,3in\n")
        if base == "highSpeed":
            f.write("set term 'epslatex' color size 2.7in,3in\n")
        else:
            f.write("set term 'epslatex' color size 2.7in,3in\n")
        f.write(f"set output '{base}.tex'\n")
        #f.write(f"set title '2020 Chevy Bolt Round Trip Timings Differences at {speed} mph'\n")
        f.write("set xrange [0:200]\n") # Be consistent
        f.write("set yrange [-60:60]\n") 
        f.write("set xtics 20\n")
        f.write("set ytics 10\n")
        f.write("set grid\n")

        f.write("set format x '{\\tiny %g}'\n")

        if base == "highSpeed":
            f.write("set xlabel 'Distance (miles)'\n")
            f.write("set key left top\n")
            f.write("set ylabel 'Time (minutes)'\n")
            f.write("set ytics\n")
        else:
            f.write("set xlabel ' '\n")
            f.write("unset key\n")
            f.write("set ylabel ' '\n")
            f.write("set format y '  '\n")

        f.write("set style line 1 lw 4 lc 'red' lt 1\n")
        f.write("set style line 2 lw 4 lc 'blue' lt 1 dt 2\n")
        f.write("set style line 3 lw 4 lc 'green' lt 1 dt 3\n")
        f.write("set style line 4 lw 4 lc 'purple' lt 1 dt 4\n")
        f.write("plot '-' using 1:2 w lines ls 2 title '$\\Delta$Total Time',\\\n")
        f.write("   '-' using 1:2 w lines ls 3 title '$\\Delta$Travel Time', '-' using 1:2 w lines ls 4 title '$\\Delta$Charge Time'\n")

        for travelDistance, totalTimeDiff in zip(travelDistances, totalTimesDiff):
            f.write(f"{travelDistance} {totalTimeDiff}\n")

        f.write("e\n")

        for travelDistance, travelTimeDiff in zip(travelDistances, travelTimesDiff):
            f.write(f"{travelDistance} {travelTimeDiff}\n")

        f.write("e\n")

        for travelDistance, chargeTimeDiff in zip(travelDistances, chargeTimesDiff):
            f.write(f"{travelDistance} {chargeTimeDiff}\n")

        f.write("e\n")

def PlotSpeedVersusSpacing(path="highSpeedSpacing.gnuplot", SInitial=1.0, SMargin=0.15, speed=80.0):
    C, maxCapacity, maxRange = Bolt(year=2020)
    S, T, dT = IntegrateOverSOC(C, maxCapacity)

    base=os.path.splitext(os.path.basename(path))[0]
    travelDistance = 1500

    chargeStopOverhead = 10.0/60.0

    # Total time spent traveling at speed v to the destination
    def TravelTime(m):
        return travelDistances[m]/speed

    # This is the power consumed to travel to stop m
    def TravelPower(m):
        return D(speed) * TravelTime(m)

    # This is the SOC upon arriving at stop m
    def DestinationSOC(m):
        return ChargeToSOC(m) - TravelPower(m)/maxCapacity

    # This is the SOC needed to go to stop m
    def ChargeToSOC(m):
        if m == 0: # Since you have a fixed SOC when leaving home, we just return initial SOC to reach stop 0
            return SInitial

        # Cover a corner case where arrival SOC at stop m-1 is larger than the SOC needed to go to stop m
        # Cover a corner case where you need to charge to SOC 115% (80 mph case)
        return min(1.0, max(SMargin + TravelPower(m)/maxCapacity, DestinationSOC(m-1)))

    # This is the time needed to charge at stop m in order to reach stop m+1
    def ChargeTime(m):
        value = np.interp(ChargeToSOC(m+1), S, T) - np.interp(DestinationSOC(m), S, T)

        if value > 0.0:
            return value + chargeStopOverhead

        return 0.0

    def TotalTime():
        value = 0.0

        # Sum over road segments
        for m in range(len(travelDistances)):
            value += TravelTime(m)

        # Sum over stops (1 less)
        for m in range(len(travelDistances)-1):
            value += ChargeTime(m)

        return value

    spacings = []
    totalTimesDiff = []
    travelTimesDiff = []
    chargeTimesDiff = []
    totalTimesRelDiff = []

    for spacing in np.linspace(25, 175, 31):
        fullStops = int(travelDistance // spacing)

        remainder = travelDistance - fullStops * spacing

        travelDistances = [spacing]*fullStops

        if remainder > 0:
            travelDistances += [remainder]

        travelDistances = np.array(travelDistances)

        # NOTE: This travelDistances affects the inner functions above!

        totalTimeFixed = TotalTime()
        travelTimeFixed = np.array([ TravelTime(m) for m in range(len(travelDistances)) ])
        chargeTimeFixed = np.array([ ChargeTime(m) for m in range(len(travelDistances)-1) ])
        destinationSOCFixed = DestinationSOC(0)

        v, totalTime, travelTime, chargeTime, destinationSOC, chargeToSOC = SolveForShortestTimeMultipleStops(travelDistances, maxCapacity, S, T, dT, SInitial=SInitial, SMargin=SMargin, chargeStopOverhead=chargeStopOverhead)

        if destinationSOCFixed < 0.0 or destinationSOC[0] < 0.0: # Can't reach the first stop at this spacing?
            break

        spacings.append(spacing)
        totalTimesDiff.append(totalTimeFixed - totalTime)
        travelTimesDiff.append(travelTimeFixed.sum() - travelTime.sum())
        chargeTimesDiff.append(chargeTimeFixed.sum() - chargeTime.sum())
        totalTimesRelDiff.append((totalTimeFixed - totalTime)/totalTime if totalTime > 0.0 else 0.0)

    print(f"Maximum relative multi-stop time difference for {speed} mph: {max(totalTimesRelDiff)}")

    with open(path, mode="wt", newline="") as f:
        if base == "highSpeedSpacing":
            f.write("set term 'epslatex' color size 2.7in,3in\n")
        else:
            f.write("set term 'epslatex' color size 2.7in,3in\n")
        f.write(f"set output '{base}.tex'\n")
        #f.write(f"set title '2020 Chevy Bolt Multi-Stop Trip Timing Differences at {speed} mph'\n")
        f.write("set xlabel 'Distance per Stop (miles)'\n")
        f.write("set xrange [20:180]\n") # Be consistent
        f.write("set yrange [-8:8]\n")
        f.write("set xtics 20\n")
        f.write("set ytics 1\n")
        f.write("set ylabel 'Time (hours)'\n")
        f.write("set grid\n")

        f.write("set format x '{\\tiny %g}'\n")

        if base == "highSpeedSpacing":
            f.write("set xlabel 'Distance per Stop (miles)'\n")
            f.write("set key left bottom\n")
            f.write("set ylabel 'Time (hours)'\n")
            f.write("set ytics\n")
        else:
            f.write("set xlabel ' '\n")
            f.write("unset key\n")
            f.write("set ylabel ' '\n")
            f.write("set format y '  '\n")

        f.write("set style line 1 lw 4 lc 'red' lt 1\n")
        f.write("set style line 2 lw 4 lc 'blue' lt 1 dt 2\n")
        f.write("set style line 3 lw 4 lc 'green' lt 1 dt 3\n")
        f.write("set style line 4 lw 4 lc 'purple' lt 1 dt 4\n")
        f.write("plot '-' using 1:2 w lines ls 2 title '$\\Delta$Total Time',\\\n")
        f.write("   '-' using 1:2 w lines ls 3 title '$\\Delta$Travel Time', '-' using 1:2 w lines ls 4 title '$\\Delta$Charge Time'\n")

        for spacing, totalTimeDiff in zip(spacings, totalTimesDiff):
            f.write(f"{spacing} {totalTimeDiff}\n")

        f.write("e\n")

        for spacing, travelTimeDiff in zip(spacings, travelTimesDiff):
            f.write(f"{spacing} {travelTimeDiff}\n")

        f.write("e\n")

        for spacing, chargeTimeDiff in zip(spacings, chargeTimesDiff):
            f.write(f"{spacing} {chargeTimeDiff}\n")

        f.write("e\n")

if __name__ == "__main__":
    PlotBoltChargeCurves()

    PlotOptimalSpeedVersusDistance(SInitial=0.8, SMargin=0.15)
    PlotOptimalSpeedVersusSpacing(SInitial=1.0, SMargin=0.15)

    PlotOptimalSpeedSOC(SInitial=0.8, SMargin=0.15)
    PlotOptimalSpeedVersusSpacingSOC(SInitial=1.0, SMargin=0.15)

    PlotSpeedVersusDistance(SInitial=0.8, SMargin=0.15, speed=80.0)
    PlotSpeedVersusDistance(path="lowSpeed.gnuplot", SInitial=0.8, SMargin=0.15, speed=60.0)

    PlotSpeedVersusSpacing(SInitial=1.0, SMargin=0.15, speed=80.0)
    PlotSpeedVersusSpacing(path="lowSpeedSpacing.gnuplot", SInitial=1.0, SMargin=0.15, speed=60.0)

